//cpu8040_disasm.c
//disassemble instructions

#include <stdio.h>
#include "cpu8040_disasm.h"

using namespace xatremu;

//execute next instruction in program memory
void CPU8040::DisAsmNextInstruction(void) {
	if (r_PC < ROMLen) {
		(this->* CPUFuncTable[GetInstrMem(r_PC)])();

	};
};

CPU8040::CPU8040() {

//00-0f
CPUFuncTable[0] = &CPU8040::I_NOP;
CPUFuncTable[1] = &CPU8040::I_AJMP_CODE;
CPUFuncTable[2] = &CPU8040::I_LJMP_CODE;
CPUFuncTable[3] = &CPU8040::I_RR_A;
CPUFuncTable[4] = &CPU8040::I_INC_A;
CPUFuncTable[5] = &CPU8040::I_INC_DATA;
CPUFuncTable[6] = &CPU8040::I_INC_AT_R0;
CPUFuncTable[7] = &CPU8040::I_INC_AT_R1;
CPUFuncTable[8] = &CPU8040::I_INC_R0;
CPUFuncTable[9] = &CPU8040::I_INC_R1;
CPUFuncTable[0xa] = &CPU8040::I_INC_R2;
CPUFuncTable[0xb] = &CPU8040::I_INC_R3;
CPUFuncTable[0xc] = &CPU8040::I_INC_R4;
CPUFuncTable[0xd] = &CPU8040::I_INC_R5;
CPUFuncTable[0xe] = &CPU8040::I_INC_R6;
CPUFuncTable[0xf] = &CPU8040::I_INC_R7;

//10-1f
CPUFuncTable[0x10] = &CPU8040::I_JBC_BIT_CODE;
CPUFuncTable[0x11] = &CPU8040::I_ACALL_CODE;
CPUFuncTable[0x12] = &CPU8040::I_LCALL_CODE;
CPUFuncTable[0x13] = &CPU8040::I_RRC_A;
CPUFuncTable[0x14] = &CPU8040::I_DEC_A;
CPUFuncTable[0x15] = &CPU8040::I_DEC_DATA;
CPUFuncTable[0x16] = &CPU8040::I_DEC_AT_R0;
CPUFuncTable[0x17] = &CPU8040::I_DEC_AT_R1;
CPUFuncTable[0x18] = &CPU8040::I_DEC_R0;
CPUFuncTable[0x19] = &CPU8040::I_DEC_R1;
CPUFuncTable[0x1a] = &CPU8040::I_DEC_R2;
CPUFuncTable[0x1b] = &CPU8040::I_DEC_R3;
CPUFuncTable[0x1c] = &CPU8040::I_DEC_R4;
CPUFuncTable[0x1d] = &CPU8040::I_DEC_R5;
CPUFuncTable[0x1e] = &CPU8040::I_DEC_R6;
CPUFuncTable[0x1f] = &CPU8040::I_DEC_R7;

//20-2f
CPUFuncTable[0x20] = &CPU8040::I_JB_BIT_CODE;
CPUFuncTable[0x21] = &CPU8040::I_AJMP_CODE;
CPUFuncTable[0x22] = &CPU8040::I_RET;
CPUFuncTable[0x23] = &CPU8040::I_RL_A;
CPUFuncTable[0x24] = &CPU8040::I_ADD_A_IMM;
CPUFuncTable[0x25] = &CPU8040::I_ADD_A_DATA;
CPUFuncTable[0x26] = &CPU8040::I_ADD_A_AT_R0;
CPUFuncTable[0x27] = &CPU8040::I_ADD_A_AT_R1;
CPUFuncTable[0x28] = &CPU8040::I_ADD_A_R0;
CPUFuncTable[0x29] = &CPU8040::I_ADD_A_R1;
CPUFuncTable[0x2a] = &CPU8040::I_ADD_A_R2;
CPUFuncTable[0x2b] = &CPU8040::I_ADD_A_R3;
CPUFuncTable[0x2c] = &CPU8040::I_ADD_A_R4;
CPUFuncTable[0x2d] = &CPU8040::I_ADD_A_R5;
CPUFuncTable[0x2e] = &CPU8040::I_ADD_A_R6;
CPUFuncTable[0x2f] = &CPU8040::I_ADD_A_R7;

//30-3f
CPUFuncTable[0x30] = &CPU8040::I_JNB_BIT_CODE;
CPUFuncTable[0x31] = &CPU8040::I_ACALL_CODE;
CPUFuncTable[0x32] = &CPU8040::I_RETI;
CPUFuncTable[0x33] = &CPU8040::I_RLC_A;
CPUFuncTable[0x34] = &CPU8040::I_ADDC_A_IMM;
CPUFuncTable[0x35] = &CPU8040::I_ADDC_A_DATA;
CPUFuncTable[0x36] = &CPU8040::I_ADDC_A_AT_R0;
CPUFuncTable[0x37] = &CPU8040::I_ADDC_A_AT_R1;
CPUFuncTable[0x38] = &CPU8040::I_ADDC_A_R0;
CPUFuncTable[0x39] = &CPU8040::I_ADDC_A_R1;
CPUFuncTable[0x3a] = &CPU8040::I_ADDC_A_R2;
CPUFuncTable[0x3b] = &CPU8040::I_ADDC_A_R3;
CPUFuncTable[0x3c] = &CPU8040::I_ADDC_A_R4;
CPUFuncTable[0x3d] = &CPU8040::I_ADDC_A_R5;
CPUFuncTable[0x3e] = &CPU8040::I_ADDC_A_R6;
CPUFuncTable[0x3f] = &CPU8040::I_ADDC_A_R7;

//40-4f
CPUFuncTable[0x40] = &CPU8040::I_JC_CODE;
CPUFuncTable[0x41] = &CPU8040::I_AJMP_CODE;
CPUFuncTable[0x42] = &CPU8040::I_ORL_DATA_A;
CPUFuncTable[0x43] = &CPU8040::I_ORL_DATA_IMM;
CPUFuncTable[0x44] = &CPU8040::I_ORL_A_IMM;
CPUFuncTable[0x45] = &CPU8040::I_ORL_A_DATA;
CPUFuncTable[0x46] = &CPU8040::I_ORL_A_AT_R0;
CPUFuncTable[0x47] = &CPU8040::I_ORL_A_AT_R1;
CPUFuncTable[0x48] = &CPU8040::I_ORL_A_R0;
CPUFuncTable[0x49] = &CPU8040::I_ORL_A_R1;
CPUFuncTable[0x4a] = &CPU8040::I_ORL_A_R2;
CPUFuncTable[0x4b] = &CPU8040::I_ORL_A_R3;
CPUFuncTable[0x4c] = &CPU8040::I_ORL_A_R4;
CPUFuncTable[0x4d] = &CPU8040::I_ORL_A_R5;
CPUFuncTable[0x4e] = &CPU8040::I_ORL_A_R6;
CPUFuncTable[0x4f] = &CPU8040::I_ORL_A_R7;

//50-5f
CPUFuncTable[0x50] = &CPU8040::I_JNC_CODE;
CPUFuncTable[0x51] = &CPU8040::I_ACALL_CODE;
CPUFuncTable[0x52] = &CPU8040::I_ANL_DATA_A;
CPUFuncTable[0x53] = &CPU8040::I_ANL_DATA_IMM;
CPUFuncTable[0x54] = &CPU8040::I_ANL_A_IMM;
CPUFuncTable[0x55] = &CPU8040::I_ANL_A_DATA;
CPUFuncTable[0x56] = &CPU8040::I_ANL_A_AT_R0;
CPUFuncTable[0x57] = &CPU8040::I_ANL_A_AT_R1;
CPUFuncTable[0x58] = &CPU8040::I_ANL_A_R0;
CPUFuncTable[0x59] = &CPU8040::I_ANL_A_R1;
CPUFuncTable[0x5a] = &CPU8040::I_ANL_A_R2;
CPUFuncTable[0x5b] = &CPU8040::I_ANL_A_R3;
CPUFuncTable[0x5c] = &CPU8040::I_ANL_A_R4;
CPUFuncTable[0x5d] = &CPU8040::I_ANL_A_R5;
CPUFuncTable[0x5e] = &CPU8040::I_ANL_A_R6;
CPUFuncTable[0x5f] = &CPU8040::I_ANL_A_R7;

//60-6f
CPUFuncTable[0x60] = &CPU8040::I_JZ_CODE;
CPUFuncTable[0x61] = &CPU8040::I_AJMP_CODE;
CPUFuncTable[0x62] = &CPU8040::I_XRL_DATA_A;
CPUFuncTable[0x63] = &CPU8040::I_XRL_DATA_IMM;
CPUFuncTable[0x64] = &CPU8040::I_XRL_A_IMM;
CPUFuncTable[0x65] = &CPU8040::I_XRL_A_DATA;
CPUFuncTable[0x66] = &CPU8040::I_XRL_A_AT_R0;
CPUFuncTable[0x67] = &CPU8040::I_XRL_A_AT_R1;
CPUFuncTable[0x68] = &CPU8040::I_XRL_A_R0;
CPUFuncTable[0x69] = &CPU8040::I_XRL_A_R1;
CPUFuncTable[0x6a] = &CPU8040::I_XRL_A_R2;
CPUFuncTable[0x6b] = &CPU8040::I_XRL_A_R3;
CPUFuncTable[0x6c] = &CPU8040::I_XRL_A_R4;
CPUFuncTable[0x6d] = &CPU8040::I_XRL_A_R5;
CPUFuncTable[0x6e] = &CPU8040::I_XRL_A_R6;
CPUFuncTable[0x6f] = &CPU8040::I_XRL_A_R7;

//70-7f
CPUFuncTable[0x70] = &CPU8040::I_JNZ_CODE;
CPUFuncTable[0x71] = &CPU8040::I_ACALL_CODE;
CPUFuncTable[0x72] = &CPU8040::I_ORL_C_BIT;
CPUFuncTable[0x73] = &CPU8040::I_JMP_AT_A_WITH_DPTR;
CPUFuncTable[0x74] = &CPU8040::I_MOV_A_IMM;
CPUFuncTable[0x75] = &CPU8040::I_MOV_DATA_IMM;
CPUFuncTable[0x76] = &CPU8040::I_MOV_AT_R0_IMM;
CPUFuncTable[0x77] = &CPU8040::I_MOV_AT_R1_IMM;
CPUFuncTable[0x78] = &CPU8040::I_MOV_R0_IMM;
CPUFuncTable[0x79] = &CPU8040::I_MOV_R1_IMM;
CPUFuncTable[0x7a] = &CPU8040::I_MOV_R2_IMM;
CPUFuncTable[0x7b] = &CPU8040::I_MOV_R3_IMM;
CPUFuncTable[0x7c] = &CPU8040::I_MOV_R4_IMM;
CPUFuncTable[0x7d] = &CPU8040::I_MOV_R5_IMM;
CPUFuncTable[0x7e] = &CPU8040::I_MOV_R6_IMM;
CPUFuncTable[0x7f] = &CPU8040::I_MOV_R7_IMM;

//80-8f
CPUFuncTable[0x80] = &CPU8040::I_SJMP_CODE;
CPUFuncTable[0x81] = &CPU8040::I_AJMP_CODE;
CPUFuncTable[0x82] = &CPU8040::I_ANL_C_BIT;
CPUFuncTable[0x83] = &CPU8040::I_MOVC_A_AT_A_WITH_PC;
CPUFuncTable[0x84] = &CPU8040::I_DIV_AB;
CPUFuncTable[0x85] = &CPU8040::I_MOV_DATA_DATA;
CPUFuncTable[0x86] = &CPU8040::I_MOV_DATA_AT_R0;
CPUFuncTable[0x87] = &CPU8040::I_MOV_DATA_AT_R1;
CPUFuncTable[0x88] = &CPU8040::I_MOV_DATA_R0;
CPUFuncTable[0x89] = &CPU8040::I_MOV_DATA_R1;
CPUFuncTable[0x8a] = &CPU8040::I_MOV_DATA_R2;
CPUFuncTable[0x8b] = &CPU8040::I_MOV_DATA_R3;
CPUFuncTable[0x8c] = &CPU8040::I_MOV_DATA_R4;
CPUFuncTable[0x8d] = &CPU8040::I_MOV_DATA_R5;
CPUFuncTable[0x8e] = &CPU8040::I_MOV_DATA_R6;
CPUFuncTable[0x8f] = &CPU8040::I_MOV_DATA_R7;

//90-9f
CPUFuncTable[0x90] = &CPU8040::I_MOV_DPTR_IMM;
CPUFuncTable[0x91] = &CPU8040::I_ACALL_CODE;
CPUFuncTable[0x92] = &CPU8040::I_MOV_BIT_C;
CPUFuncTable[0x93] = &CPU8040::I_MOVC_A_AT_A_WITH_DPTR;
CPUFuncTable[0x94] = &CPU8040::I_SUBB_A_IMM;
CPUFuncTable[0x95] = &CPU8040::I_SUBB_A_DATA;
CPUFuncTable[0x96] = &CPU8040::I_SUBB_A_AT_R0;
CPUFuncTable[0x97] = &CPU8040::I_SUBB_A_AT_R1;
CPUFuncTable[0x98] = &CPU8040::I_SUBB_A_R0;
CPUFuncTable[0x99] = &CPU8040::I_SUBB_A_R1;
CPUFuncTable[0x9a] = &CPU8040::I_SUBB_A_R2;
CPUFuncTable[0x9b] = &CPU8040::I_SUBB_A_R3;
CPUFuncTable[0x9c] = &CPU8040::I_SUBB_A_R4;
CPUFuncTable[0x9d] = &CPU8040::I_SUBB_A_R5;
CPUFuncTable[0x9e] = &CPU8040::I_SUBB_A_R6;
CPUFuncTable[0x9f] = &CPU8040::I_SUBB_A_R7;

//a0-af
CPUFuncTable[0xa0] = &CPU8040::I_ORL_C_SLBIT;
CPUFuncTable[0xa1] = &CPU8040::I_AJMP_CODE;
CPUFuncTable[0xa2] = &CPU8040::I_MOV_C_BIT;
CPUFuncTable[0xa3] = &CPU8040::I_INC_DPTR;
CPUFuncTable[0xa4] = &CPU8040::I_MUL_AB;
CPUFuncTable[0xa5] = &CPU8040::I_RES;
CPUFuncTable[0xa6] = &CPU8040::I_MOV_AT_R0_DATA;
CPUFuncTable[0xa7] = &CPU8040::I_MOV_AT_R1_DATA;
CPUFuncTable[0xa8] = &CPU8040::I_MOV_R0_DATA;
CPUFuncTable[0xa9] = &CPU8040::I_MOV_R1_DATA;
CPUFuncTable[0xaa] = &CPU8040::I_MOV_R2_DATA;
CPUFuncTable[0xab] = &CPU8040::I_MOV_R3_DATA;
CPUFuncTable[0xac] = &CPU8040::I_MOV_R4_DATA;
CPUFuncTable[0xad] = &CPU8040::I_MOV_R5_DATA;
CPUFuncTable[0xae] = &CPU8040::I_MOV_R6_DATA;
CPUFuncTable[0xaf] = &CPU8040::I_MOV_R7_DATA;

//b0-bf
CPUFuncTable[0xb0] = &CPU8040::I_ANL_C_SLBIT;
CPUFuncTable[0xb1] = &CPU8040::I_ACALL_CODE;
CPUFuncTable[0xb2] = &CPU8040::I_CPL_BIT;
CPUFuncTable[0xb3] = &CPU8040::I_CPL_C;
CPUFuncTable[0xb4] = &CPU8040::I_CJNE_A_IMM_CODE;
CPUFuncTable[0xb5] = &CPU8040::I_CJNE_A_DATA_CODE;
CPUFuncTable[0xb6] = &CPU8040::I_CJNE_AT_R0_IMM_CODE;
CPUFuncTable[0xb7] = &CPU8040::I_CJNE_AT_R1_IMM_CODE;
CPUFuncTable[0xb8] = &CPU8040::I_CJNE_R0_IMM_CODE;
CPUFuncTable[0xb9] = &CPU8040::I_CJNE_R1_IMM_CODE;
CPUFuncTable[0xba] = &CPU8040::I_CJNE_R2_IMM_CODE;
CPUFuncTable[0xbb] = &CPU8040::I_CJNE_R3_IMM_CODE;
CPUFuncTable[0xbc] = &CPU8040::I_CJNE_R4_IMM_CODE;
CPUFuncTable[0xbd] = &CPU8040::I_CJNE_R5_IMM_CODE;
CPUFuncTable[0xbe] = &CPU8040::I_CJNE_R6_IMM_CODE;
CPUFuncTable[0xbf] = &CPU8040::I_CJNE_R7_IMM_CODE;

//c0-cf
CPUFuncTable[0xc0] = &CPU8040::I_PUSH_DATA;
CPUFuncTable[0xc1] = &CPU8040::I_AJMP_CODE;
CPUFuncTable[0xc2] = &CPU8040::I_CLR_BIT;
CPUFuncTable[0xc3] = &CPU8040::I_CLR_C;
CPUFuncTable[0xc4] = &CPU8040::I_SWAP_A;
CPUFuncTable[0xc5] = &CPU8040::I_XCH_A_DATA;
CPUFuncTable[0xc6] = &CPU8040::I_XCH_A_AT_R0;
CPUFuncTable[0xc7] = &CPU8040::I_XCH_A_AT_R1;
CPUFuncTable[0xc8] = &CPU8040::I_XCH_A_R0;
CPUFuncTable[0xc9] = &CPU8040::I_XCH_A_R1;
CPUFuncTable[0xca] = &CPU8040::I_XCH_A_R2;
CPUFuncTable[0xcb] = &CPU8040::I_XCH_A_R3;
CPUFuncTable[0xcc] = &CPU8040::I_XCH_A_R4;
CPUFuncTable[0xcd] = &CPU8040::I_XCH_A_R5;
CPUFuncTable[0xce] = &CPU8040::I_XCH_A_R6;
CPUFuncTable[0xcf] = &CPU8040::I_XCH_A_R7;

//d0-df
CPUFuncTable[0xd0] = &CPU8040::I_POP_DATA;
CPUFuncTable[0xd1] = &CPU8040::I_ACALL_CODE;
CPUFuncTable[0xd2] = &CPU8040::I_SETB_BIT;
CPUFuncTable[0xd3] = &CPU8040::I_SETB_C;
CPUFuncTable[0xd4] = &CPU8040::I_DA_A;
CPUFuncTable[0xd5] = &CPU8040::I_DJNZ_DATA_CODE;
CPUFuncTable[0xd6] = &CPU8040::I_XCHD_A_AT_R0;
CPUFuncTable[0xd7] = &CPU8040::I_XCHD_A_AT_R1;
CPUFuncTable[0xd8] = &CPU8040::I_DJNZ_R0_CODE;
CPUFuncTable[0xd9] = &CPU8040::I_DJNZ_R1_CODE;
CPUFuncTable[0xda] = &CPU8040::I_DJNZ_R2_CODE;
CPUFuncTable[0xdb] = &CPU8040::I_DJNZ_R3_CODE;
CPUFuncTable[0xdc] = &CPU8040::I_DJNZ_R4_CODE;
CPUFuncTable[0xdd] = &CPU8040::I_DJNZ_R5_CODE;
CPUFuncTable[0xde] = &CPU8040::I_DJNZ_R6_CODE;
CPUFuncTable[0xdf] = &CPU8040::I_DJNZ_R7_CODE;

//e0-ef
CPUFuncTable[0xe0] = &CPU8040::I_MOVX_A_AT_DPTR;
CPUFuncTable[0xe1] = &CPU8040::I_AJMP_CODE;
CPUFuncTable[0xe2] = &CPU8040::I_MOVX_A_AT_R0;
CPUFuncTable[0xe3] = &CPU8040::I_MOVX_A_AT_R1;
CPUFuncTable[0xe4] = &CPU8040::I_CLR_A;
CPUFuncTable[0xe5] = &CPU8040::I_MOV_A_DATA;
CPUFuncTable[0xe6] = &CPU8040::I_MOV_A_AT_R0;
CPUFuncTable[0xe7] = &CPU8040::I_MOV_A_AT_R1;
CPUFuncTable[0xe8] = &CPU8040::I_MOV_A_R0;
CPUFuncTable[0xe9] = &CPU8040::I_MOV_A_R1;
CPUFuncTable[0xea] = &CPU8040::I_MOV_A_R2;
CPUFuncTable[0xeb] = &CPU8040::I_MOV_A_R3;
CPUFuncTable[0xec] = &CPU8040::I_MOV_A_R4;
CPUFuncTable[0xed] = &CPU8040::I_MOV_A_R5;
CPUFuncTable[0xee] = &CPU8040::I_MOV_A_R6;
CPUFuncTable[0xef] = &CPU8040::I_MOV_A_R7;

//f0-ff
CPUFuncTable[0xf0] = &CPU8040::I_MOVX_AT_DPTR_A;
CPUFuncTable[0xf1] = &CPU8040::I_ACALL_CODE;
CPUFuncTable[0xf2] = &CPU8040::I_MOVX_AT_R0_A;
CPUFuncTable[0xf3] = &CPU8040::I_MOVX_AT_R1_A;
CPUFuncTable[0xf4] = &CPU8040::I_CPL_A;
CPUFuncTable[0xf5] = &CPU8040::I_MOV_DATA_A;
CPUFuncTable[0xf6] = &CPU8040::I_MOV_AT_R0_A;
CPUFuncTable[0xf7] = &CPU8040::I_MOV_AT_R1_A;
CPUFuncTable[0xf8] = &CPU8040::I_MOV_R0_A;
CPUFuncTable[0xf9] = &CPU8040::I_MOV_R1_A;
CPUFuncTable[0xfa] = &CPU8040::I_MOV_R2_A;
CPUFuncTable[0xfb] = &CPU8040::I_MOV_R3_A;
CPUFuncTable[0xfc] = &CPU8040::I_MOV_R4_A;
CPUFuncTable[0xfd] = &CPU8040::I_MOV_R5_A;
CPUFuncTable[0xfe] = &CPU8040::I_MOV_R6_A;
CPUFuncTable[0xff] = &CPU8040::I_MOV_R7_A;

};

CPU8040::~CPU8040() {
};

//private:
//	unsigned short r_PC;
//	unsigned char r_R0, r_R1, r_R2, r_R3, r_R4, r_R5, r_R6, r_R7;
//	unsigned int64 totalCycles;
//	unsigned char r_A, r_B;
//	bool r_C;

void CPU8040::I_ACALL_CODE(void) {
	unsigned short int a = 0;
	a = ((GetInstrMem(r_PC) & 0xe0) << 3) + GetInstrMem(r_PC);
	printf("%04x\tACALL\t%04x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_ADD_A_AT_R0(void) {
	printf("%04x\tADD\tA,@R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_ADD_A_AT_R1(void) {
	printf("%04x\tADD\tA,@R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_ADD_A_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tADD\tA,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_ADD_A_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tADD\tA,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_ADD_A_R0(void) {
	printf("%04x\tADD\tA,R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_ADD_A_R1(void) {
        printf("%04x\tADD\tA,R1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADD_A_R2(void) {
        printf("%04x\tADD\tA,R2\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADD_A_R3(void) {
        printf("%04x\tADD\tA,R3\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADD_A_R4(void) {
        printf("%04x\tADD\tA,R4\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADD_A_R5(void) {
        printf("%04x\tADD\tA,R5\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADD_A_R6(void) {
        printf("%04x\tADD\tA,R6\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADD_A_R7(void) {
        printf("%04x\tADD\tA,R7\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_AT_R0(void) {
        printf("%04x\tADDC\tA,@R0\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_AT_R1(void) {
        printf("%04x\tADDC\tA,@R1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_DATA(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tADD\tA,%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_ADDC_A_IMM(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tADD\tA,#%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_ADDC_A_R0(void) {
        printf("%04x\tADDC\tA,R0\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_R1(void) {
       printf("%04x\tADDC\tA,R1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_R2(void) {
       printf("%04x\tADDC\tA,R2\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_R3(void) {
       printf("%04x\tADDC\tA,R3\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_R4(void) {
       printf("%04x\tADDC\tA,R4\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_R5(void) {
       printf("%04x\tADDC\tA,R5\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_R6(void) {
       printf("%04x\tADDC\tA,R6\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ADDC_A_R7(void) {
       printf("%04x\tADDC\tA,R7\n", r_PC);
        ++r_PC;
};

void CPU8040::I_AJMP_CODE(void) {
        unsigned short int a = 0;
        a = ((GetInstrMem(r_PC) & 0xe0) << 3) + GetInstrMem(r_PC);
        printf("%04x\tAJMP\t%04x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_ANL_A_AT_R0(void) {
        printf("%04x\tANL\tA,@R0\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_AT_R1(void) {
        printf("%04x\tANL\tA,@R1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_DATA(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tANL\tA,%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_ANL_A_IMM(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tANL\tA,#%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_ANL_A_R0(void) {
        printf("%04x\tANL\tA,R0\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_R1(void) {
        printf("%04x\tANL\tA,R1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_R2(void) {
        printf("%04x\tANL\tA,R2\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_R3(void) {
        printf("%04x\tANL\tA,R3\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_R4(void) {
        printf("%04x\tANL\tA,R4\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_R5(void) {
        printf("%04x\tANL\tA,R5\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_R6(void) {
        printf("%04x\tANL\tA,R6\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_A_R7(void) {
        printf("%04x\tANL\tA,R7\n", r_PC);
        ++r_PC;
};

void CPU8040::I_ANL_DATA_A(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tANL\t%02x,A\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_ANL_DATA_IMM(void) {
        unsigned short int a = 0;
	unsigned short int b = 0;
        a = GetInstrMem(r_PC + 1);
	b = GetInstrMem(r_PC + 2);
        printf("%04x\tANL\t%02x,#%02x\n", r_PC, a, b);
        r_PC = r_PC + 2;
};

void CPU8040::I_ANL_C_BIT(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tANL\tC,P%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_ANL_C_SLBIT(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tANL\tC,/P%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_CJNE_A_DATA_CODE(void) {
	unsigned short a = 0;
	signed char b = 0;
	unsigned short int c = 0;
	a = GetInstrMem(r_PC + 1);
	b = GetInstrMem(r_PC + 2);
	c = b + r_PC + 3;
	printf("%04x\tCJNE\tA,%02x,%04x\n", r_PC, a, c);
	r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_A_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tA,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_R0_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tR0,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_R1_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tR1,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_R2_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tR2,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_R3_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tR3,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_R4_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tR4,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_R5_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tR5,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_R6_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tR6,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_R7_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\tR7,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_AT_R0_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\t@R0,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CJNE_AT_R1_IMM_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tCJNE\t@R1,#%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_CLR_A(void) {
	printf("%04x\tCLR\tA\n", r_PC);
	++r_PC;
};

void CPU8040::I_CLR_BIT(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tCLR\tP%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_CLR_C(void) {
        printf("%04x\tCLR\tC\n", r_PC);
        ++r_PC;
};

void CPU8040::I_CPL_A(void) {
        printf("%04x\tCPL\tA\n", r_PC);
        ++r_PC;
};

void CPU8040::I_CPL_BIT(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tCPL\tP%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_CPL_C(void) {
        printf("%04x\tCPL\tC\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DA_A(void) {
        printf("%04x\tDA\tA\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_A(void) {
        printf("%04x\tDEC\tA\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_AT_R0(void) {
        printf("%04x\tDEC\t@R0\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_AT_R1(void) {
        printf("%04x\tDEC\t@R1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_DATA(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tDEC\t%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_DEC_R0(void) {
        printf("%04x\tDEC\tR0\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_R1(void) {
        printf("%04x\tDEC\tR1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_R2(void) {
        printf("%04x\tDEC\tR2\n", r_PC);
        ++r_PC;
};
void CPU8040::I_DEC_R3(void) {
        printf("%04x\tDEC\tR3\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_R4(void) {
        printf("%04x\tDEC\tR4\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_R5(void) {
        printf("%04x\tDEC\tR5\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_R6(void) {
        printf("%04x\tDEC\tR6\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DEC_R7(void) {
        printf("%04x\tDEC\tR7\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DIV_AB(void) {
        printf("%04x\tDIV\tAB\n", r_PC);
        ++r_PC;
};

void CPU8040::I_DJNZ_DATA_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tDJNZ\t%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_DJNZ_R0_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tDJNZ\tR0,%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_DJNZ_R1_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tDJNZ\tR1,%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_DJNZ_R2_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tDJNZ\tR2,%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_DJNZ_R3_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tDJNZ\tR3,%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_DJNZ_R4_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tDJNZ\tR4,%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_DJNZ_R5_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tDJNZ\tR5,%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_DJNZ_R6_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tDJNZ\tR6,%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_DJNZ_R7_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tDJNZ\tR7,%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_INC_A(void) {
        printf("%04x\tINC\tA\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_AT_R0(void) {
        printf("%04x\tINC\t@R0\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_AT_R1(void) {
        printf("%04x\tINC\t@R1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_DATA(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tINC\t%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_INC_DPTR(void) {
        printf("%04x\tINC\tDPTR\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_R0(void) {
        printf("%04x\tINC\tR0\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_R1(void) {
        printf("%04x\tINC\tR1\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_R2(void) {
        printf("%04x\tINC\tR2\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_R3(void) {
        printf("%04x\tINC\tR3\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_R4(void) {
        printf("%04x\tINC\tR4\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_R5(void) {
        printf("%04x\tINC\tR5\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_R6(void) {
        printf("%04x\tINC\tR6\n", r_PC);
        ++r_PC;
};

void CPU8040::I_INC_R7(void) {
        printf("%04x\tINC\tR7\n", r_PC);
        ++r_PC;
};

void CPU8040::I_JB_BIT_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tJB\tP%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_JBC_BIT_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tJBC\tP%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_JC_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tJC\t%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_JMP_AT_A_WITH_DPTR(void) {
        printf("%04x\tJMP\t@A+DPTR\n", r_PC);
        ++r_PC;
};

void CPU8040::I_JNB_BIT_CODE(void) {
        unsigned short a = 0;
        signed char b = 0;
        unsigned short int c = 0;
        a = GetInstrMem(r_PC + 1);
        b = GetInstrMem(r_PC + 2);
        c = b + r_PC + 3;
        printf("%04x\tJNB\tP%02x,%04x\n", r_PC, a, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_JNC_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tJNC\t%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_JNZ_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tJNZ\t%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_JZ_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tJZ\t%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_LCALL_CODE(void) {
        unsigned short int c = 0;
        c = (GetInstrMem(r_PC + 1) << 8) + GetInstrMem(r_PC + 2);
        printf("%04x\tLCALL\t%04x\n", r_PC, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_LJMP_CODE(void) {
        unsigned short int c = 0;
        c = (GetInstrMem(r_PC + 1) << 8) + GetInstrMem(r_PC + 2);
        printf("%04x\tLJMP\t%04x\n", r_PC, c);
        r_PC = r_PC + 3;
};

void CPU8040::I_MOV_A_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
        printf("%04x\tMOV\tA,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_A_AT_R0(void) {
	printf("%04x\tMOV\tA,@R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_AT_R1(void) {
	printf("%04x\tMOV\tA,@R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tA,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_A_R0(void) {
	printf("%04x\tMOV\tA,R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_R1(void) {
	printf("%04x\tMOV\tA,R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_R2(void) {
	printf("%04x\tMOV\tA,R2\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_R3(void) {
	printf("%04x\tMOV\tA,R3\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_R4(void) {
	printf("%04x\tMOV\tA,R4\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_R5(void) {
	printf("%04x\tMOV\tA,R5\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_R6(void) {
	printf("%04x\tMOV\tA,R6\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_A_R7(void) {
	printf("%04x\tMOV\tA,R7\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_AT_R0_A(void) {
	printf("%04x\tMOV\t@R0,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_AT_R1_A(void) {
	printf("%04x\tMOV\t@R1,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_AT_R0_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t@R0,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_AT_R1_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t@R1,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_AT_R0_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t@R0,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_AT_R1_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t@R1,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_BIT_C(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tMOV\tP%02x,C\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_MOV_C_BIT(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tMOV\tC,P%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_A(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,A\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_AT_R0(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,@R0\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_AT_R1(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,@R1\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_DATA(void) {
	unsigned short a = 0;
	unsigned short int c = 0;
	a = GetInstrMem(r_PC + 1);
	c = GetInstrMem(r_PC + 2);
	printf("%04x\tMOV\t%02x,%02x\n", r_PC, a, c);
	r_PC = r_PC + 3;
};

void CPU8040::I_MOV_DATA_IMM(void) {
	unsigned short a = 0;
	unsigned short int c = 0;
	a = GetInstrMem(r_PC + 1);
	c = GetInstrMem(r_PC + 2);
	printf("%04x\tMOV\t%02x,#%02x\n", r_PC, a, c);
	r_PC = r_PC + 3;
};

void CPU8040::I_MOV_DATA_R0(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,R0\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_R1(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,R1\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_R2(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,R2\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_R3(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,R3\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_R4(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,R4\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_R5(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,R5\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_R6(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,R6\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DATA_R7(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\t%02x,R7\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_DPTR_IMM(void) {
	unsigned short a = 0;
	unsigned short int c = 0;
	a = GetInstrMem(r_PC + 1);
	c = GetInstrMem(r_PC + 2);
	printf("%04x\tMOV\tDPTR,#%02x%02x\n", r_PC, a, c);
	r_PC = r_PC + 3;
};

void CPU8040::I_MOV_R0_A(void) {
	printf("%04x\tMOV\tR0,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_R1_A(void) {
	printf("%04x\tMOV\tR1,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_R2_A(void) {
	printf("%04x\tMOV\tR2,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_R3_A(void) {
	printf("%04x\tMOV\tR3,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_R4_A(void) {
	printf("%04x\tMOV\tR4,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_R5_A(void) {
	printf("%04x\tMOV\tR5,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_R6_A(void) {
	printf("%04x\tMOV\tR6,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOV_R7_A(void) {
	static int z = 0;
	if ((GetInstrMem(r_PC + 1) == 255) && (GetInstrMem(r_PC + 2) == 255)) {
		z = 1;
	};
	if (!z) {
		printf("%04x\tMOV\tR7,A\n", r_PC);
	} else {
		printf("%04x\tDC.B\tFF\n", r_PC);
	};
	if (GetInstrMem(r_PC + 1) != 255) {
		z = 0;
	};
	++r_PC;
};

void CPU8040::I_MOV_R0_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR0,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R1_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR1,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R2_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR2,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R3_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR3,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R4_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR4,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R5_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR5,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R6_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR6,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R7_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR7,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R0_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR0,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R1_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR1,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R2_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR2,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R3_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR3,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R4_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR4,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R5_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR5,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R6_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR6,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOV_R7_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tMOV\tR7,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_MOVC_A_AT_A_WITH_DPTR(void) {
	printf("%04x\tMOVC\tA,@A+DPTR\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOVC_A_AT_A_WITH_PC(void) {
	printf("%04x\tMOVC\tA,@A+PC\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOVX_A_AT_DPTR(void) {
	printf("%04x\tMOVX\tA,@DPTR\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOVX_A_AT_R0(void) {
	printf("%04x\tMOVX\tA,@R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOVX_A_AT_R1(void) {
	printf("%04x\tMOVX\tA,@R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOVX_AT_DPTR_A(void) {
	printf("%04x\tMOVX\t@DPTR,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOVX_AT_R0_A(void) {
	printf("%04x\tMOVX\t@R0,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MOVX_AT_R1_A(void) {
	printf("%04x\tMOVX\t@R1,A\n", r_PC);
	++r_PC;
};

void CPU8040::I_MUL_AB(void) {
	printf("%04x\tMUL\tAB\n", r_PC);
	++r_PC;
};

void CPU8040::I_NOP(void) {
	printf("%04x\tNOP\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_AT_R0(void) {
	printf("%04x\tORL\tA,@R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_AT_R1(void) {
	printf("%04x\tORL\tA,@R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tORL\tA,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_ORL_A_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tORL\tA,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_ORL_A_R0(void) {
	printf("%04x\tORL\tA,R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_R1(void) {
	printf("%04x\tORL\tA,R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_R2(void) {
	printf("%04x\tORL\tA,R2\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_R3(void) {
	printf("%04x\tORL\tA,R3\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_R4(void) {
	printf("%04x\tORL\tA,R4\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_R5(void) {
	printf("%04x\tORL\tA,R5\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_R6(void) {
	printf("%04x\tORL\tA,R6\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_A_R7(void) {
	printf("%04x\tORL\tA,R7\n", r_PC);
	++r_PC;
};

void CPU8040::I_ORL_DATA_A(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tORL\t%02x,A\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_ORL_DATA_IMM(void) {
	unsigned short a = 0;
	unsigned short int c = 0;
	a = GetInstrMem(r_PC + 1);
	c = GetInstrMem(r_PC + 2);
	printf("%04x\tORL\t%02x,#%02x\n", r_PC, a, c);
	r_PC = r_PC + 3;
};

void CPU8040::I_ORL_C_BIT(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tORL\tC,P%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_ORL_C_SLBIT(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tORL\tC,/P%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_POP_DATA(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tPOP\t%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_PUSH_DATA(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tPUSH\t%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_RES(void) { //reserved
	printf("%04x\tRES\n", r_PC);
	++r_PC;
};

void CPU8040::I_RET(void) {
	printf("%04x\tRET\n", r_PC);
	++r_PC;
};

void CPU8040::I_RETI(void) {
	printf("%04x\tRETI\n", r_PC);
	++r_PC;
};

void CPU8040::I_RL_A(void) {
	printf("%04x\tRL\tA\n", r_PC);
	++r_PC;
};

void CPU8040::I_RLC_A(void) {
	printf("%04x\tRLC\tA\n", r_PC);
	++r_PC;
};

void CPU8040::I_RR_A(void) {
	printf("%04x\tRR\tA\n", r_PC);
	++r_PC;
};

void CPU8040::I_RRC_A(void) {
	printf("%04x\tRRC\tA\n", r_PC);
	++r_PC;
};

void CPU8040::I_SETB_BIT(void) {
        unsigned short int a = 0;
        a = GetInstrMem(r_PC + 1);
        printf("%04x\tSETB\tP%02x\n", r_PC, a);
        r_PC = r_PC + 2;
};

void CPU8040::I_SETB_C(void) {
	printf("%04x\tSETB\tC\n", r_PC);
	++r_PC;
};

void CPU8040::I_SJMP_CODE(void) {
        signed char b = 0;
        unsigned short int c = 0;
        b = GetInstrMem(r_PC + 1);
        c = b + r_PC + 2;
        printf("%04x\tSJMP\t%04x\n", r_PC, c);
        r_PC = r_PC + 2;
};

void CPU8040::I_SUBB_A_AT_R0(void) {
	printf("%04x\tSUBB\tA,@R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_AT_R1(void) {
	printf("%04x\tSUBB\tA,@R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tSUBB\tA,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_SUBB_A_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tSUBB\tA,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_SUBB_A_R0(void) {
	printf("%04x\tSUBB\tA,R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_R1(void) {
	printf("%04x\tSUBB\tA,R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_R2(void) {
	printf("%04x\tSUBB\tA,R2\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_R3(void) {
	printf("%04x\tSUBB\tA,R3\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_R4(void) {
	printf("%04x\tSUBB\tA,R4\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_R5(void) {
	printf("%04x\tSUBB\tA,R5\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_R6(void) {
	printf("%04x\tSUBB\tA,R6\n", r_PC);
	++r_PC;
};

void CPU8040::I_SUBB_A_R7(void) {
	printf("%04x\tSUBB\tA,R7\n", r_PC);
	++r_PC;
};

void CPU8040::I_SWAP_A(void) {
	printf("%04x\tSWAP\tA\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_AT_R0(void) {
	printf("%04x\tXCH\tA,@R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_AT_R1(void) {
	printf("%04x\tXCH\tA,@R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tXCH\tA,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_XCH_A_R0(void) {
	printf("%04x\tXCH\tA,R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_R1(void) {
	printf("%04x\tXCH\tA,R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_R2(void) {
	printf("%04x\tXCH\tA,R2\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_R3(void) {
	printf("%04x\tXCH\tA,R3\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_R4(void) {
	printf("%04x\tXCH\tA,R4\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_R5(void) {
	printf("%04x\tXCH\tA,R5\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_R6(void) {
	printf("%04x\tXCH\tA,R6\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCH_A_R7(void) {
	printf("%04x\tXCH\tA,R7\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCHD_A_AT_R0(void) {
	printf("%04x\tXCHD\tA,@R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_XCHD_A_AT_R1(void) {
	printf("%04x\tXCHD\tA,@R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_AT_R0(void) {
	printf("%04x\tXRL\tA,@R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_AT_R1(void) {
	printf("%04x\tXRL\tA,@R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_DATA(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tXRL\tA,%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_XRL_A_IMM(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tXRL\tA,#%02x\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_XRL_A_R0(void) {
	printf("%04x\tXRL\tA,R0\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_R1(void) {
	printf("%04x\tXRL\tA,R1\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_R2(void) {
	printf("%04x\tXRL\tA,R2\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_R3(void) {
	printf("%04x\tXRL\tA,R3\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_R4(void) {
	printf("%04x\tXRL\tA,R4\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_R5(void) {
	printf("%04x\tXRL\tA,R5\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_R6(void) {
	printf("%04x\tXRL\tA,R6\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_A_R7(void) {
	printf("%04x\tXRL\tA,R7\n", r_PC);
	++r_PC;
};

void CPU8040::I_XRL_DATA_A(void) {
	unsigned short int a = 0;
	a = GetInstrMem(r_PC + 1);
	printf("%04x\tXRL\t%02x,A\n", r_PC, a);
	r_PC = r_PC + 2;
};

void CPU8040::I_XRL_DATA_IMM(void) {
	unsigned short a = 0;
	unsigned short int c = 0;
	a = GetInstrMem(r_PC + 1);
	c = GetInstrMem(r_PC + 2);
	printf("%04x\tXRL\t%02x,#%02x\n", r_PC, a, c);
	r_PC = r_PC + 3;
};

//set a value in data memory
//void CPU8040::SetDataMem(unsigned short addr, unsigned char value);
//get a value from data memory
//unsigned char CPU8040::GetDataMem(unsigned short addr);

//set instruction Mem Pointer
void CPU8040::SetInstrMemPtr(void* ptr) {
	instrMemPtr = ptr;
};

//set ROM length
void CPU8040::SetROMLen(unsigned long myLen) {
	ROMLen = myLen;
};

//set a value in instruction memory
//void CPU8040::SetInstrMem(unsigned short addr, unsigned char value); //for testing only

//set instruction pointer
void CPU8040::SetInstrPtr(unsigned short myPtr) {
	r_PC = myPtr;
};

//get instruction pointer
unsigned short CPU8040::GetInstrPtr() {
	return r_PC;
};

//get a value from instruction memory
unsigned char CPU8040::GetInstrMem(unsigned short addr) {
//	return (unsigned char) *((unsigned short*) ((unsigned char *) instrMemPtr + addr));
//	return (unsigned char) *( (unsigned char *) instrMemPtr + addr);
	return *( (unsigned char *) instrMemPtr + addr);
};

	//get/set io-area
//void CPU8040::SetIOArea(unsigned char bit, bool value);
//bool CPU8040::GetIOArea(unsigned char bit);

