//serial_real.c
// management of serial data, cpu-bound, real thing 

#include <stdio.h>
#include "serial_real.h"

namespace xatremu
{

SerialReal::SerialReal(){
};

SerialReal::~SerialReal(){
};

void SerialReal::ResetSerial(){
	m_currentInBit=10;
	m_currentOutBit=11;
	m_CPU8040->SetNextSpecialEvent(CPUEVENTSERIALIN, MAXCYCLE);
	m_CPU8040->SetNextSpecialEvent(CPUEVENTSERIALOUT, MAXCYCLE);
	m_stateIntLine=true;
};

//Set state of interrupt line for 8040 as given by Atari
bool SerialReal::StateIntLine(){
	return m_stateIntLine;
};

//Atari: Sets interrupt line (to low when SKCTL & 0x70 == 0x20)
void SerialReal::SetIntLineAtari(bool intLine){
	m_stateIntLine = intLine;
/*	if(!intLine)
		m_CPU8040->RunCPUForCycles(60);
*/
};

//set state of T0 line for 8040
bool SerialReal::StateT0Line(){
	if (m_currentInBit == 0)
		return false;
	if ((m_currentInBit >= 1) && (m_currentInBit <= 8)) {
		return ((m_currentInByte >> (m_currentInBit - 1)) & 0x01);
	};
	if(m_currentInBit == 9)
		return true;

	return true;
};

//event handling: next bit of T0 for 8040
void SerialReal::T0NextBit(){
	if (m_currentInBit < 10)
		++m_currentInBit;
	if (m_currentInBit < 10)
		m_CPU8040->SetNextSpecialEvent(CPUEVENTSERIALIN, m_transferbitcycles);
#ifdef TESTSTATE
	printf("BitFromA800: number %d is %d from %2x\n", m_currentInBit, StateT0Line(), m_currentInByte);
#endif
};

//get state of Port 1.7 from 8040
void SerialReal::SerialOutBit(bool outBit){
	m_storedOutBit = outBit;
	if((m_currentOutBit == 11) && (outBit == true)){
		m_transferbitcycles = m_bitcycles;
		m_transferwaitcycles = m_waitcycles;
		//set the timing of the start bit a bit faster than normal
		m_CPU8040->SetNextSpecialEvent(CPUEVENTSERIALOUT, m_bitcycles-2);
		m_currentOutBit = 0;
	};
	if(m_currentOutBit == 9){
		m_CPU8040->DelayCPU(m_transferwaitcycles);
		m_CPU8040->SetNextSpecialEvent(CPUEVENTSERIALOUT, 2);
		m_currentOutBit = 10;
	};
};

//event handling: next bit of Port 1.7 from 8040
void SerialReal::P17NextBit(){
	if(m_currentOutBit == 10){
		m_currentOutBit = 11;
		m_storedOutByte = m_currentOutByte;
		m_storedOutByteOk = true;
#ifdef TESTSTATE
		printf("Stored OutByte: %2x from %4x\n", m_storedOutByte, m_CPU8040->GetInstrPtr());
#endif
	};
	if((m_currentOutBit >= 1) && (m_currentOutBit <= 8)){
		m_currentOutByte = (m_currentOutByte >> 1) + (m_storedOutBit ? 0 : 0x80);
		m_CPU8040->SetNextSpecialEvent(CPUEVENTSERIALOUT, m_transferbitcycles);
		++m_currentOutBit;
	};
	if(m_currentOutBit == 0){
		if(m_storedOutBit == true){
			m_CPU8040->SetNextSpecialEvent(CPUEVENTSERIALOUT, m_transferbitcycles);
			++m_currentOutBit;
		}
		else {
			m_currentOutBit = 11;
		};
	};
};

//for Atari: Floppy is ready for next input byte
bool SerialReal::ReadyForNextInByte(){
	return (m_currentInBit >= 10);
};

//Atari sends next input byte to floppy
//shall only be called when ReadyForNextInputByte returned true
void SerialReal::SerialByteFromAtari(unsigned char A800Byte){
	m_transferbitcycles = m_bitcycles;
	m_transferwaitcycles = m_waitcycles;
	m_currentInByte = A800Byte;
	m_CPU8040->DelayCPU(m_transferwaitcycles);
	m_currentInBit = -1;
	m_CPU8040->SetNextSpecialEvent(CPUEVENTSERIALIN, m_bitcycles-1);
};

//for Atari: Floppy has completed sending an out byte
bool SerialReal::SerialOutByteComplete() {
	if (!m_storedOutByteOk) return false;
	m_storedOutByteOk = false;
	return true;
};

//Atari can get next byte from floppy
//shall only be called when SerialOutByteComplete returned true
unsigned char SerialReal::SerialByteToAtari(){
	return m_storedOutByte;
};

//check SIO speed and adjust communications
void SerialReal::SetSIOSpeed(unsigned char Speed){
	if(Speed == 0x28){
		m_bitcycles = 28;
		m_waitcycles = 3906 - m_bitcycles * 9;
	}
	else if(Speed == 0x10){
		m_bitcycles = 14;
		m_waitcycles = 1953 - m_bitcycles * 9;
	}
	else{
		m_bitcycles = 36;
		m_waitcycles = 4800;
	};
};

}
