//wd1772_real.h
// management of WD1772 data, real chip

#ifndef __XATREMU_WD1772_REAL__
#define __XATREMU_WD1772_REAL__

#include "wd1772_base.h"

namespace xatremu
{
class WD1772Real : public WD1772Base
{
public:
	WD1772Real();
	virtual ~WD1772Real();
	virtual void ResetWD1772();
	virtual void ChooseDoubleDensity(bool doubleDensity);
	virtual void MasterReset();
	virtual void ChooseRegister(unsigned char reg);
	virtual void ChooseReadRegisters(bool readRegs);
	virtual void ChooseSide(bool side);
	virtual bool SetImage(char* imageName);
	virtual void EjectImage();
	virtual void Choose288RPM(bool rpm288);
        virtual bool GetIntRq();
        virtual bool StateT1Line(); //data request line
	virtual void CheckDRQEvent();
	virtual void CheckHeadStepDelay();
	virtual void CheckHeadCalmDelay();
	virtual void CheckMotor();
	virtual void DataSearchTimeOut();
	virtual void MotorOn();
	virtual void MotorOff();
	virtual void PrepareMotorOff();
	virtual void CountTrackDataOffset();
        virtual unsigned char ByteFromWD1772();
        virtual void ByteToWD1772(unsigned char myByte);
	virtual unsigned char GetStatusRegister();
	virtual void CreateTrackData();
	virtual void CreateTrackFromATR();
	virtual unsigned int EncodeByteFM(unsigned char byteToEncode);
	virtual unsigned char DecodeByteFM(unsigned int byteToDecode);
	virtual unsigned int EncodeByteMFM(unsigned char byteToEncode, bool previousBit);
	virtual unsigned char DecodeByteMFM(unsigned int byteToDecode);
	virtual unsigned int CalculateCRC(unsigned int CRCStart, unsigned char byteToAdd);
	virtual unsigned int CalculateCRCEnd(unsigned int CRCStart);
	virtual void SetTrackDataWord(unsigned char * trackPointer, unsigned int dataWord);
	virtual unsigned long int GetTrackDataLongword(unsigned char * trackStartPointer, unsigned int dataOffset);
	virtual unsigned int CyclesToNextDataWord();
	virtual signed char SearchSync(unsigned long int myDataLongWord, unsigned int mySync1, unsigned int mySync2);
#ifdef WITH_CAPS
	virtual char AnalyzeFromCT();
	virtual unsigned char DecodeByteFMCT(unsigned long byteToDecode);
	virtual unsigned int RecodeDataFMCT(unsigned long recodeLongWord);
	virtual uint64_t GetTrackDataQuadWordCT(char currentRevolution, unsigned int dataOffset);
	virtual signed char SearchSyncsCTFM(uint64_t myDataQuadWord, unsigned long mySync1, unsigned long mySync2, unsigned long mySync3);
#endif

protected:
	typedef void (WD1772Real::* WD1772Func)(void);
	WD1772Func m_MotorFunc;
	WD1772Func m_DataFunc;
	enum WD1772State {
		wd1772_idle = 0,	//wd1772 is idle
		wd1772_motoridle,	//only motor running and waiting for 9 revolves
		wd1772_restore,		//restore (seek track 0) in progress
		wd1772_stepin,		//stepping inside
		wd1772_readsearchheadersync,	//searching the header sync
		wd1772_readgotgenericsyncforheader,	//generic MFM sync found, header expected
		wd1772_readgotheadersync,	//got header sync
		wd1772_readsearchdatasync,	//searching the sync of data field
		wd1772_readgotdatasync,	//generic MFM sync found, data expected
		wd1772_readgetdata,	//get data from disk
		wd1772_readcheckdatacrc,	//read and check data crc
		wd1772_rasearchheadersync,	//search header sync for read address
		wd1772_ragotgenericsyncforheader,	//generic MFM sync found for read address
		wd1772_ragotheadersync,	//got header sync for read address
		wd1772_seek		//seek command issued
	};

private:
/*
	bool m_IsDoubleDensity;
	unsigned char m_SelectedRegister; // which address (0-3) is selected
	bool m_SelectRegsForRead;	// true - read, false - write
	bool m_IsDiskSide1;	//false: side0, true: side1
	unsigned char m_CurrentTrack; //0-39
	bool m_Is288rpm;	// for compatibility, create a WD1772 for 288RPM too
	unsigned short int m_EncodedBytesPerTrack; //3255 for FM as standard

	//formula: (is288rpm ? 0.208333333 : 0.2) * 8333333.333 / bytesPerTrack
	unsigned short m_CpuCyclesToNextEncodedByteOfTrack;
*/	
	WD1772State m_WD1772State;

	friend class testCPU8040;
};
}
#endif
